@@ -1,6 +1,10 @@
 #-*- coding:utf-8 -*-
 import json
+import multiprocessing
+import platform
+import threading
 
+from multiprocessing import Process
 from retrying import retry
 
 __author__ = 'cc'
@@ -11,9 +15,9 @@ import redis
 import time
 import queue
 import traceback
+from loguru import logger
 from collections import Callable
-from concurrent.futures import ThreadPoolExecutor, Future
-from concurrent.futures.thread import _WorkItem
+from concurrent.futures import ThreadPoolExecutor
 from tomorrow3 import threads as tomorrow_threads
 
 # redis配置连接信息
@@ -24,6 +28,7 @@ redis_db = 0
 
 redis_conn_instance = {}
 
+
 class RedisQueue(object):
     """Simple Queue with Redis Backend"""
 
@@ -93,70 +98,96 @@ class RedisQueue(object):
 class RedisCustomer(object):
     """reids队列消费类"""
 
-    def __init__(self, queue_name, consuming_function: Callable = None, threads_num=50,max_retry_times=3):
+    def __init__(self, queue_name, consuming_function: Callable = None,process_num=1,threads_num=50,max_retry_times=3,is_support_mutil_param=False):
         """
         redis队列消费程序
         :param queue_name: 队列名称
         :param consuming_function: 队列消息取出来后执行的方法
         :param threads_num: 启动多少个队列线程
         :param max_retry_times: 错误重试次数
+        :param is_support_mutil_param: 消费函数是否支持多个参数,默认False
         """
-        self.redis_quenen = RedisQueue(queue_name, host=redis_host, port=redis_port, db=redis_db,
+        self._redis_quenen = RedisQueue(queue_name, host=redis_host, port=redis_port, db=redis_db,
                                        password=redis_password)
-        self.consuming_function = consuming_function
+        self._consuming_function = consuming_function
+        self.queue_name = queue_name
+        self.process_num = process_num
         self.threads_num = threads_num
-        self.threadpool = BoundedThreadPoolExecutor(threads_num)
+        self._threadpool = BoundedThreadPoolExecutor(threads_num)
         self.max_retry_times = max_retry_times
+        self.is_support_mutil_param = is_support_mutil_param
 
-    def start_consuming_message(self):
-        print('*' * 50)
+    def _start_consuming_message_thread(self):
+        logger.info(f'start consuming message mutil_thread, threads_num:{self.threads_num}')
         while True:
             try:
-                message = self.redis_quenen.get()
+                message = self._redis_quenen.get()
                 if message:
-                    self.threadpool.submit(self._consuming_exception_retry, message)
+                    if self.is_support_mutil_param:
+                       message = json.loads(message)
+                       if type(message)!=dict:
+                          raise Exception('请发布【字典】类型消息,当前消息是【字符串】类型')
+                    self._threadpool.submit(self._consuming_exception_retry, message)
                 else:
                     time.sleep(0.1)
             except:
                 s = traceback.format_exc()
-                print(s)
+                logger.error(s)
                 time.sleep(0.1)
 
+    def start_consuming_message(self):
+        cpu_count = multiprocessing.cpu_count()
+        logger.info(f'start consuming message  mutil_process,process_num:{min(self.process_num,cpu_count)},system:{platform.system()}')
+        if platform.system()=='Darwin' or platform.system()=='Linux':
+            for i in range(0,min(self.process_num,cpu_count)):
+                Process(target=self._start_consuming_message_thread).start()
+        else:
+            threading.Thread(target=self._start_consuming_message_thread).start()
+
     def _consuming_exception_retry(self,message):
         @retry(stop_max_attempt_number=self.max_retry_times)
         def consuming_exception_retry(message):
-            self.consuming_function(message)
+            if type(message)==dict:
+                self._consuming_function(**message)
+            else:
+                self._consuming_function(message)
         consuming_exception_retry(message)
 
 
-
 class RedisPublish(object):
     """redis入队列类"""
 
-    def __init__(self, queue_name, fliter_rep=False, max_push_size=1):
+    def __init__(self, queue_name, fliter_rep=False, max_push_size=50):
         """
         初始化消息发布队列
         :param queue_name: 队列名称(不包含命名空间)
         :param fliter_rep: 队列任务是否去重 True:去重  False:不去重
-        :param threads_num: 并发线程数
-        :param max_push_size: 
+        :param max_push_size: 使用批量提交时,每次批量提交数量
         """
-        self.redis_quenen = RedisQueue(queue_name,fliter_rep=fliter_rep, host=redis_host, port=redis_port, db=redis_db,
+        self._redis_quenen = RedisQueue(queue_name,fliter_rep=fliter_rep, host=redis_host, port=redis_port, db=redis_db,
                                        password=redis_password)
-        # self.threads_num = threads_num
+        self.queue_name = queue_name
         self.max_push_size = max_push_size
-        self.local_quenen = queue.Queue(maxsize=max_push_size + 1)
-        self.pipe = self.redis_quenen.getdb().pipeline()
+        self._local_quenen = queue.Queue(maxsize=max_push_size + 1)
+        self._pipe = self._redis_quenen.getdb().pipeline()
 
     @tomorrow_threads(50)
-    def publish_redispy(self, **kwargs):
+    def publish_redispy(self,*args,**kwargs):
         """
         将多参数写入消息队列
         :param kwargs: 待写入参数 (a=3,b=4)
         :return: None
         """
-        dict_msg = dict(sorted(kwargs.items(), key=lambda d: d[0]))
-        self.redis_quenen.put(json.dumps(dict_msg))
+        # logger.info(f"args:{args},kwargs:{kwargs}")
+        dict_msg = None
+        if kwargs:
+            dict_msg = dict(sorted(kwargs.items(), key=lambda d: d[0]))
+        elif args:
+            dict_msg = args[0]
+        else:
+            logger.warning('参数非法')
+        if dict_msg:
+            self._redis_quenen.put(json.dumps(dict_msg))
 
     @tomorrow_threads(50)
     def publish_redispy_str(self, msg:str):
@@ -165,21 +196,20 @@ class RedisPublish(object):
         :param msg: 待写入消息字符串
         :return: None
         """
-        self.redis_quenen.put(msg)
+        self._redis_quenen.put(msg)
 
-    def publish_redispy_list(self, msgs: list, publish_size=50):
+    def publish_redispy_list(self, msgs: list):
         """
         批量写入redis队列
         :param msgs: 待写入字符串列表
-        :param publish_size: 每次批量提交数量
         :return: 
         """
-        pipe = self.redis_quenen.getdb().pipeline()
+        pipe = self._redis_quenen.getdb().pipeline()
         for id in msgs:
-            pipe.lpush(self.redis_quenen.key, id)
-            if len(pipe) == publish_size:
+            pipe.lpush(self._redis_quenen.key, id)
+            if len(pipe) == self.max_push_size:
                 pipe.execute()
-                print(f'*' * 10 + str(publish_size) + '*' * 10 + 'commit')
+                logger.info(str(self.max_push_size).center(20,'*') + 'commit')
         if len(pipe)>0:
             pipe.execute()
 
@@ -189,23 +219,23 @@ class RedisPublish(object):
         :param msg: 待写入字符串
         :return: None
         """
-        self.local_quenen.put(msg)
-        print(f'self.local_quenen.size:{self.local_quenen.qsize()}')
-        if self.local_quenen.qsize() >= self.max_push_size:
+        self._local_quenen.put(msg)
+        # logger.info(f'self._local_quenen.size:{self._local_quenen.qsize()}')
+        if self._local_quenen.qsize() >= self.max_push_size:
             try:
-                while self.local_quenen.qsize() > 0:
-                    self.pipe.lpush(self.redis_quenen.key, self.local_quenen.get_nowait())
+                while self._local_quenen.qsize() > 0:
+                    self._pipe.lpush(self._redis_quenen.key, self._local_quenen.get_nowait())
             except:
-                traceback.print_exc()
-            self.pipe.execute()
-            print('commit'.center(16, '*'))
+                logger.error(traceback.format_exc())
+            self._pipe.execute()
+            logger.info('commit'.center(16, '*'))
 
     def clear_quenen(self):
         """
         清空当前队列
         :return: 
         """
-        self.redis_quenen.clear()
+        self._redis_quenen.clear()
 
 
 class BoundedThreadPoolExecutor(ThreadPoolExecutor):
@@ -214,55 +244,54 @@ class BoundedThreadPoolExecutor(ThreadPoolExecutor):
         self._work_queue = queue.Queue(max_workers * 1)
 
     def submit(self, fn, *args, **kwargs):
-        with self._shutdown_lock:
-            if self._shutdown:
-                raise RuntimeError('cannot schedule new futures after shutdown')
-            f = Future()
-            fn_deco = _deco(fn)
-            w = _WorkItem(f, fn_deco, args, kwargs)
-            self._work_queue.put(w)
-            self._adjust_thread_count()
-            return f
+        fn_deco = self._deco(fn)
+        super().submit(fn_deco, *args, **kwargs)
 
-
-def _deco(f):
-    @wraps(f)
-    def __deco(*args, **kwargs):
-        try:
-            return f(*args, **kwargs)
-        except Exception as e:
-            print(e)
-
-    return __deco
+    def _deco(self,f):
+        @wraps(f)
+        def __deco(*args, **kwargs):
+            try:
+                return f(*args, **kwargs)
+            except Exception as e:
+                logger.error(e)
+        return __deco
 
 
 if __name__ == '__main__':
     redis_host = '127.0.0.1'
     redis_password = ''
     redis_port = 6379
-    redis_db = 0
+    redis_db = 8
 
     quenen_name = 'test1'
-    redis_pub = RedisPublish(queue_name=quenen_name,fliter_rep=True, max_push_size=50)
+    # 初始化发布队列 fliter_rep=True任务自动去重
+    redis_pub = RedisPublish(queue_name=quenen_name,fliter_rep=False, max_push_size=50)
 
-    result = [str(i) for i in range(1, 101)]
+    result = [str(i) for i in range(1, 501)]
+
+    def print_msg_str(msg):
+        print(f"msg_str:{msg}")
 
     for zz in result:
-        redis_pub.publish_redispy(c=zz, b=zz, a=zz)  # 单条记录写入
+        redis_pub.publish_redispy_str(zz)  # 写入字符串任务
 
-    redis_pub.publish_redispy_list(result)  # 单线程提交列表任务
-
-    for zz in result:
-        redis_pub.publish_redispy_mutil(zz)  # 单线程批量写入2
-
-
-    def print_msg(msg):
-        print(json.loads(msg))
-        # raise Exception('hand exception 123')
-
-
-    # 多线程消费
-    redis_customer = RedisCustomer(quenen_name, consuming_function=print_msg, threads_num=100,max_retry_times=5)
-    print(redis_customer.threads_num)
+    # 多线程消费字符串任务
+    redis_customer = RedisCustomer(quenen_name, consuming_function=print_msg_str, process_num=2, threads_num=100,
+                                   max_retry_times=5)
     redis_customer.start_consuming_message()
 
+    for zz in result:
+        redis_pub.publish_redispy(c=zz, b=zz, a=zz)  # 写入字典任务 {"c":zz,"b":zz,"a":zz}
+
+    # redis_pub.publish_redispy_list(result)  # 批量提交任务1
+
+    def print_msg_dict(a,b,c):
+        print(f"msg_dict:{a},{b},{c}")
+
+    # 多线程消费字典任务
+    redis_customer = RedisCustomer(quenen_name, consuming_function=print_msg_dict,process_num=2,threads_num=100,
+                                   max_retry_times=5,is_support_mutil_param=True)
+    redis_customer.start_consuming_message()
+
+
+
